# -*- mode: cperl; coding: utf-8-unix; indent-tabs-mode: t -*-
# $Id: $

use Cwd 'cwd';
use ExtUtils::MakeMaker;
use strict;

# QUICK INSTALLATION
#
# $ env URL_BASE=http://www.yourdomain.com/eshoppy_mall \
# EMAIL=yourname@yourdomain.com \
# DBI_DBTYPE=Pg \
# DBI_DBNAME=eshoppy4 \
# DBI_USER=db_username \
# DBI_PASS=db_password \
# PERL_MM_USE_DEFAULT=1 \
# /usr/bin/perl Makefile.PL && make && make fetch && make install

my $MYHOST = $ENV{MYHOST}||`hostname`; chomp $MYHOST;
my $DIR_BASE = prompt('current directory: ', $ENV{DIR_BASE}||cwd);
my $DIR_LOCAL_IN = prompt('local direcotry: ', -e 'local' ? './local' : $DIR_BASE);
my $DIR_LOCAL = $DIR_LOCAL_IN; $DIR_LOCAL =~ s/^\./$DIR_BASE/;
my ($CWD) = $DIR_BASE =~ /\/([^\/]+)$/;
my $URL_BASE = prompt('current url: ', $ENV{URL_BASE}||"http://$MYHOST/$CWD");
my $DIR_STATIC = "$DIR_BASE/static";
my $DIR_IMAGE = "$DIR_STATIC/image";
my $DIR_MANAGE = "$DIR_BASE/manage";
my $DIR_BACKUPS = "$DIR_LOCAL/backups";
my $PG_PSQL = find_path('psql');
my $SENDMAIL = prompt('sendmail command: ', find_path('sendmail'));
my $EMAIL = prompt('your email address: ', $ENV{EMAIL}||($ENV{USER}||'info').'@'.$MYHOST); $EMAIL =~ s/@/\\@/;
my $DB_TYPE = prompt('type of the database(Pg or mysql): ', $ENV{DBI_DBTYPE}||($PG_PSQL ? 'Pg' : 'mysql'));
my $DB_ENGINE = $DB_TYPE eq 'mysql' ? prompt('storage engine for MySQL(MyISAM or InnoDB): ', $ENV{DBI_DBENGINE}||'MyISAM') : '';
my $DB_NAME = prompt('name of the database: ', $ENV{DBI_DBNAME}||'eshoppy4');
my $DB_HOST = prompt('host name for the database: ', $ENV{DBI_DBHOST}||'localhost');
my $DB_PORT = prompt('port number for the database: ', $ENV{DBI_DBPORT}||($DB_TYPE eq 'Pg' ? '5432' : '3306'));
my $DB_USER = prompt('user name for the database: ', $ENV{DBI_USER}||$ENV{USER}||'');
my $DB_PASS = prompt('password for the database: ', $ENV{DBI_PASS}||'');
my $UMASK = prompt('umask for apache user: ', $ENV{UMASK}||'000');
my $IS_SUEXEC = $UMASK eq '000' ? 0 : 1;
my ($PG_VERSION) = -e $PG_PSQL ? `$PG_PSQL --version` =~ /(\d+\.\d+)/ : (8.3);
my $USE_FULLTEXT_SEARCH = $PG_VERSION >= 8.3 ? 1 : 0;
my $MODNAME = 'ESP::SMS';

$URL_BASE =~ s/\/+$// if $URL_BASE =~ /\/+$/;
my ($URL_PATH) = $URL_BASE =~ /^https?:\/\/.+?(\/.+)$/;
my $INC_STATIC = $URL_PATH ? "$URL_PATH/static" : '/static';
$URL_PATH ? ($URL_PATH .= '/') : ($URL_PATH = '/');

my %PREREQ_PM = (
	'Cwd'         => 0,
	'DBI'         => 1.48,
	$DB_TYPE eq 'mysql' ? ('DBD::mysql' => 4.0) : ('DBD::Pg' => 1.43),
	'Encode'      => 2.10,
	'Test::More'  => 0.32,
	'Digest::MD5' => 2.33,
);

sub find_path {
	my $name = shift;
	my $env = 'PATH_'.uc $name;
	return $ENV{$env} if defined $ENV{$env};
	my $cmd;
	for my $path (split(/:/, $ENV{PATH}), qw(/sbin /usr/sbin /usr/local/sbin)) {
		-x "$path/$name" and $cmd = "$path/$name", last;
	}
	$cmd;
}

WriteMakefile(
			  'NAME'	  => 'E-Shoppy Mall',
			  'DISTNAME'  => 'eshoppy_mall',
			  'VERSION_FROM' => "local/lib/ESP/SMS/Plugin.pm", # finds $VERSION
			  'INST_LIB' => "$DIR_LOCAL/lib",
			  'INST_ARCHLIB' => "$DIR_LOCAL/arch",
			  'INST_SCRIPT' => "$DIR_LOCAL/bin",
			  'PREREQ_PM' => \%PREREQ_PM,
			  'clean' =>
			  {
			   FILES => "$DIR_LOCAL/tmp/*",
			  },

			  'test' =>
			  {
			   TESTS => "$DIR_LOCAL/t/*.t",
			  },
			  'dist' =>
			  {
			   TAR => 'tar',
			   TARFLAGS => '-c -v --owner=0 --group=0 -f',
			  },
			 );

sub MY::post_initialize {
	my($self) = shift;
	my(@m,$tmp,%cgi,%mgr,%bin,@finddirs);
	my(@loc_dirs) = qw(tmp sid tmpl tmpl/static tmpl/static/0);
	push @finddirs, $DIR_BASE;
	push @finddirs, $DIR_LOCAL if ($DIR_LOCAL !~ /^$DIR_BASE/);
	$self->{URL_BASE} = $URL_BASE;
	$self->{DIR_BASE} = $DIR_BASE;
	$self->{DIR_LOCAL} = $DIR_LOCAL;
	$self->{DIR_LOCAL_IN} = $DIR_LOCAL_IN;
	$self->{DIR_STATIC} = $DIR_STATIC;
	$self->{DIR_IMAGE} = $DIR_IMAGE;
	$self->{DIR_MANAGE} = $DIR_MANAGE;
	$self->{DIR_BACKUPS} = $DIR_BACKUPS;
	$self->{INC_STATIC} = $INC_STATIC;
	$self->{URL_PATH} = $URL_PATH;
	$self->{SENDMAIL} = $SENDMAIL;
	$self->{EMAIL} = $EMAIL;
	$self->{DB_TYPE} = $DB_TYPE;
	$self->{DB_ENGINE} = $DB_ENGINE;
	$self->{DB_NAME} = $DB_NAME;
	$self->{DB_HOST} = $DB_HOST;
	$self->{DB_PORT} = $DB_PORT;
	$self->{DB_USER} = $DB_USER;
	$self->{DB_PASS} = $DB_PASS;
	$self->{UMASK} = $UMASK;
	$self->{USE_FULLTEXT_SEARCH} = $USE_FULLTEXT_SEARCH;
	$self->{LOC_CONFIG} = '$(DIR_LOCAL)/plugins/00config.pl';
	$self->{LOC_CONFIG_IN} = '$(DIR_LOCAL)/plugins/00config.pl.default';
	$self->{LOC_CONFIG_OLD} = '$(DIR_LOCAL)/plugins/00config.pl.old';
	$self->{LOC_DIRS} = join(' ', map { '$(DIR_LOCAL)/'.$_ } @loc_dirs);
	require File::Find;
	File::Find::find(sub {
		if (-d $_){
		if ($_ eq "CVS" || $_ eq "RCS"){
			$File::Find::prune = 1;
		}
		return;
		}
		return if /\#/;
		my($path) = $File::Find::name;
		$cgi{$path} = 1 if ($path =~ /^$DIR_BASE\/\w+\.cgi$/);
		$mgr{$path} = 1 if ($path =~ /^$DIR_MANAGE\/[^\/]+\.cgi$/);
		$bin{$path} = 1 if ($path =~ /^$DIR_LOCAL\/\w+\.pl$/);
	}, @finddirs);

	$self->{CGI_FILES} = join(' ', map { /^$DIR_BASE\/(.*)/; '$(DIR_BASE)/'.$1 } keys %cgi);
	$self->{MGR_FILES} = join(' ', map { /^$DIR_MANAGE\/(.*)/; '$(DIR_MANAGE)/'.$1 } keys %mgr);
	$self->{BIN_FILES} = join(' ', map { /^$DIR_LOCAL\/(.*)/; '$(DIR_LOCAL)/'.$1 } keys %bin);
	$self->{PERM_BIN} = $IS_SUEXEC ? "700" : "700";
	$self->{PERM_CGI} = $IS_SUEXEC ? "755" : "755";
	$self->{PERM_DIR} = $IS_SUEXEC ? "755" : "777";
	$self->{PERM_DAT} = $IS_SUEXEC ? "644" : "666";
	$self->{FIND} = "find";
	$self->{ECHO} = "echo";

	for $tmp (qw/
				 DIR_BASE DIR_LOCAL DIR_LOCAL_IN DIR_STATIC DIR_IMAGE DIR_MANAGE
				 DIR_BACKUPS INC_STATIC
				 EMAIL DB_TYPE DB_ENGINE DB_NAME DB_HOST DB_PORT DB_USER DB_PASS UMASK USE_FULLTEXT_SEARCH
				 LOC_CONFIG LOC_CONFIG_IN LOC_CONFIG_OLD LOC_DIRS
				 CGI_FILES MGR_FILES BIN_FILES LOC_FILES
				 PERM_CGI PERM_BIN PERM_DIR PERM_DAT
				 FIND ECHO
				 / ) {
	next unless defined $self->{$tmp};
	push @m, "$tmp = $self->{$tmp}\n";
	}
	join('',@m);
}

sub MY::top_targets {
	my($self) = shift;
	my(@m);
	push @m, q{
all: pure_all config
};
	push @m, q{
pure_all: files
};
	push @m, q{
files: chmod_dirs chmod_files cgi_files mgr_files bin_files htaccess
};
	push @m, q{
chmod_dirs:
	$(NOECHO) $(ECHO) === Setting local directories
	$(NOECHO) $(MKPATH) $(LOC_DIRS) $(DIR_STATIC) $(DIR_IMAGE)
	-$(NOECHO) $(CHMOD) $(PERM_DIR) $(LOC_DIRS) $(DIR_STATIC) $(DIR_IMAGE)
	-$(NOECHO) $(CHMOD) 700 $(DIR_BACKUPS)
};
	push @m, q{
chmod_files:
	$(NOECHO) $(ECHO) === Setting local files
	-$(NOECHO) $(FIND) $(LOC_DIRS) -type f -exec $(CHMOD) $(PERM_DAT) {} \;
};
	push @m, q{
cgi_files:
	$(NOECHO) $(ECHO) === Setting CGI files for users
	$(NOECHO) $(PERL) -i -ne 's|^#!\S+(\s+\-\w+)*|#!}.$self->{PERL}.q{$$1|; \
		/^use lib / and s|[\w\.\/]+local|}.
		($self->{DIR_LOCAL_IN} =~ /^\./ ? $self->{DIR_LOCAL_IN} : $self->{DIR_LOCAL}).q{|g; \
		print' $(CGI_FILES)
	-$(NOECHO) $(CHMOD) $(PERM_CGI) $(CGI_FILES)
};
	push @m, q{
mgr_files:
	$(NOECHO) $(ECHO) === Setting CGI files for managers
	$(NOECHO) $(PERL) -i -ne 's|^#!\S+(\s+\-\w+)*|#!}.$self->{PERL}.q{$$1|; \
		/^use lib / and s|[\w\.\/]+local|}.
		($self->{DIR_LOCAL_IN} =~ /^\./ ? ".$self->{DIR_LOCAL_IN}" : $self->{DIR_LOCAL}).q{|g; \
		print' $(MGR_FILES)
	-$(NOECHO) $(CHMOD) $(PERM_CGI) $(MGR_FILES)
};
	push @m, q{
bin_files:
	$(NOECHO) $(ECHO) === Setting script files for managers
	$(NOECHO) $(PERL) -i -ne 's|^#!\S+(\s+\-\w+)*|#!}.$self->{PERL}.q{$$1|; \
		print' $(BIN_FILES)
	-$(NOECHO) $(CHMOD) $(PERM_BIN) $(BIN_FILES)
};
	push @m, q{
htaccess:
	$(NOECHO) $(ECHO) === Setting .htaccess files
	$(NOECHO) $(PERL) -i -ne 's|^RewriteBase .+|RewriteBase }.$self->{URL_PATH}.q{|; \
		s|^RewriteRule (..) .*?/(static/.*)|RewriteRule $$1 }.$self->{URL_PATH}.q{$$2|; print' .htaccess
};
	push @m, q{
schema:
	$(NOECHO) $(ECHO) === Setting schema for database
	$(NOECHO) $(DIR_LOCAL)/schema.pl --install
	$(NOECHO) $(ECHO) === Setting initial data for database
	$(NOECHO) $(DIR_LOCAL)/schema.pl --install_data
};
	push @m, q{
fetch:
	$(NOECHO) $(ECHO) === Getting postal code
	$(NOECHO) $(DIR_LOCAL)/postal.pl --fetch_ken_all=1
};
	push @m, q{
postal:
	$(NOECHO) $(ECHO) === Setting postal code for database
	$(NOECHO) $(DIR_LOCAL)/postal.pl
};
	push @m, q{
static_files:
	$(NOECHO) $(ECHO) === Setting static files
	$(NOECHO) $(DIR_LOCAL)/manage.pl name=webmgr pmd=build_all exec=1 sync=1 force=1 recursive=1
	-$(NOECHO) $(DIR_LOCAL)/manage.pl name=webmgr pmd=build_all exec=1 sync=0 force=1 recursive=1
};
	push @m, q{
docs:
	$(NOECHO) $(ECHO) === Creating document
	$(NOECHO) $(DIR_LOCAL)/schema.pl --output=$(DIR_LOCAL)/docs/00schema.txt
	-$(NOECHO) $(DIR_LOCAL)/docs.pl
};
	join('',@m);
}

sub MY::test_via_harness {
	my($self, $perl, $tests) = @_;
	return qq{\t$perl "-MExtUtils::Command::MM" }.
		qq{"-e" "test_harness(\$(TEST_VERBOSE), '\$(DIR_LOCAL)/plugins', '\$(INST_LIB)', '\$(DIR_LOCAL)/t/testlib')" $tests\n};
}

sub MY::install {
	my($self) = shift;
	my(@m);
	push @m, q{
install: schema postal static_files finish
};
	push @m, q#
config:
	$(NOECHO) $(ECHO) === Setting configuration file
	$(NOECHO) $(MKPATH) $(DIR_LOCAL)/plugins
	-$(NOECHO) if $(TEST_F) $(LOC_CONFIG); then $(MV) $(LOC_CONFIG) $(LOC_CONFIG_OLD); fi
	-$(NOECHO) $(PERL) -ne 's|^(\$$C->{DIR_BASE}) = (.)(?:.+)(.;)|$$1 = $${2}#.$self->{DIR_BASE}.q#$$3|; \
		s|^(\$$C->{URL_BASE}) = (.)(?:.+)(.;)|$$1 = $${2}#.$self->{URL_BASE}.q#$$3|; \
		s|^(\$$C->{INC_STATIC}) = (.)(?:.+)(.;)|$$1 = $${2}#.$self->{INC_STATIC}.q#$$3|; \
		s|^(\$$C->{CKE_PATH}) = (.)(?:.+)(.;)|$$1 = $${2}#.$self->{URL_PATH}.q#$$3|; \
		s|^(\$$C->{SENDMAIL}) = (.)(?:.+)(.;)|$$1 = $${2}#.$self->{SENDMAIL}.q#$$3|; \
		s|^(\$$C->{EMAIL}) = (.)(?:.*)(.;)|$$1 = $${2}#.$self->{EMAIL}.q#$$3|; \
		s|^(\$$C->{DB_TYPE}) = (.)(?:.*)(.;)|$$1 = $${2}#.$self->{DB_TYPE}.q#$$3|; \
		s|^(\$$C->{DB_ENGINE}) = (.)(?:.*)(.;)|$$1 = $${2}#.$self->{DB_ENGINE}.q#$$3|; \
		s|^(\$$C->{DB_NAME}) = (.)(?:.*)(.;)|$$1 = $${2}#.$self->{DB_NAME}.q#$$3|; \
		s|^(\$$C->{DB_HOST}) = (.)(?:.*)(.;)|$$1 = $${2}#.$self->{DB_HOST}.q#$$3|; \
		s|^(\$$C->{DB_PORT}) = (.)(?:.*)(.;)|$$1 = $${2}#.$self->{DB_PORT}.q#$$3|; \
		s|^(\$$C->{DB_USER}) = (.)(?:.*)(.;)|$$1 = $${2}#.$self->{DB_USER}.q#$$3|; \
		s|^(\$$C->{DB_PASS}) = (.)(?:.*)(.;)|$$1 = $${2}#.$self->{DB_PASS}.q#$$3|; \
		s|^(\$$C->{DIR_LOCAL}) = (.)(?:.*)(.;)|$$1 = $${2}#.$self->{DIR_LOCAL}.q#$$3|; \
		s|^(\$$C->{USE_FULLTEXT_SEARCH}) = (?:.+);|$$1 = #.$self->{USE_FULLTEXT_SEARCH}.q#;|; \
		s|^(\$$C->{UMASK}) = (?:.+);|$$1 = #.$self->{UMASK}.q{;|; \
		print' $(LOC_CONFIG_IN) > $(LOC_CONFIG)
	-$(NOECHO) $(PERL) -MExtUtils::Install -I$(DIR_LOCAL)/plugins -I$(DIR_LOCAL)/lib\
		-e 'require "$(LOC_CONFIG)"; }.$MODNAME.q{::Plugin::make_config()'
};
	push @m, q{
finish:
	$(NOECHO) $(PERL) -MExtUtils::Install -I$(DIR_LOCAL)/plugins -I$(DIR_LOCAL)/lib\
		-e 'require "$(LOC_CONFIG)"; }.$MODNAME.q{::Plugin::make_finish()'
};
	join('',@m);
}

__END__
# Copyright (C) 2000-2012 E-Shoppy Project developing team
# All rights reserved.
