# -*- mode: cperl; coding: utf-8-unix; indent-tabs-mode: t -*-
# $Id: $

package ESP::SMS::Plugin;

use strict;
our $VERSION = '4.1';
our $NAME = 'E-Shoppy Mall';

use Cozy::base qw(Cozy::Plugin);
use utf8;

our $C = {};

$C->{global} = [qw(NAME VERSION TIME DEMO DEBUG
				   URL_BASE URL_STATIC URL_IMAGE
				   INC_STATIC INC_IMAGE INC_STYLES INC_JS
				   CGIURL_BASE CGIURL_MANAGE COPYRIGHT
				   GOOGLE_TRACKER site_name)];

# プログラム名
$C->{NAME} = $NAME;

# プログラムバージョン
$C->{VERSION} = $VERSION;

# 必須モジュール
$C->{PREREQ_PM} = [
				   'Encode'      => 2.10,
				   'Digest::MD5' => 2.20,
				   'DBI'         => 1.48
				  ];

# オプションモジュール
$C->{OPTION_PM} = [
				   'DBD::Pg'     => 1.43,
				   'DBD::mysql'  => 4.0,
				   'Net::SMTP' => 2.29,
				   'MIME::Base64' => 3.07,
				   'Imager' => 0.42,
				   'Apache::DBI' => 1.00
				  ];

# デモ版(0=無効/1=有効)
$C->{DEMO} = 0;

# モジュール初期値
$C->{CGI}->{module} = 'Cozy::CGI';
$C->{Untaint}->{module} = 'ESP::SMS::Plugin::Untaint';
$C->{Untaint}->{config} = {
						   errkey_prefix => 'error_',
						  };
$C->{Template}->{module} = 'Cozy::Template';
$C->{Template}->{config} = {
							errkey_prefix => 'error_',
							errclass_prefix => 'error_',
						   };
$C->{DataFilter}->{module} = 'ESP::SMS::Plugin::DataFilter';
$C->{Wiki}->{module} = 'Cozy::DataFilter::Wiki';
$C->{Wiki}->{config} = {};
$C->{SafeHTML}->{module} = 'Cozy::DataFilter::SafeHTML';
$C->{SafeHTML}->{config} = {
	# ショップ管理室のファイル管理で許可するタグと属性
	tags => 'html,head,title,body,header,footer,nav,div,span,h1,h2,h3,h4,h5,h6,font,strong,b,p,br,pre,del,ul,ol,li,dl,dt,dd,table,tr,th,td,a,img,center,hr,form,input,fieldset,legend,label',
	schema => {
			   html => [qr<xmlns="[^"]+">, qr<lang="[a-z]+">, qr<xml:lang="[a-z]+">],
			   head => [],
			   title => [],
			   body => [],
			   header => [qr<class="[a-z0-9_\-\ ]+">, qr<id="[a-z0-9_\-]+">,
						  qr<data-role="[a-z0-9_\-]+">],
			   footer => [qr<class="[a-z0-9_\-\ ]+">, qr<id="[a-z0-9_\-]+">,
						  qr<data-role="[a-z0-9_\-]+">],
			   nav => [qr<class="[a-z0-9_\-\ ]+">, qr<id="[a-z0-9_\-]+">,
					   qr<data-role="[a-z0-9_\-]+">],
			   div => [qr<class="[a-z0-9_\-\ ]+">, qr<id="[a-z0-9_\-]+">, qr<align="[a-z]+">,
					   qr<data-role="[a-z0-9_\-]+">, qr<data-type="[a-z]+">,
					   qr<data-theme="[a-z]">, qr<data-content-theme="[a-z]">,
					   qr<data-collapsed="(?:true|false)">,
					   qr<data-title="[^"]+">, qr<data-id="[a-z0-9_\-]+">, qr<data-position="[a-z]+">,
					   qr<data-add-back-btn="(?:true|false)">, qr<data-back-btn-text="[^"]+">,
					   qr<data-fullscreen="(?:true|false)">, qr<style="(?:[a-z\-]+:[a-zA-Z0-9 \-#]+%?;)+">],
			   span => [qr<class="[a-z0-9_\-\ ]+">, qr<id="[a-z0-9_\-]+">,
						qr<style="(?:[a-z\-]+:[a-zA-Z0-9 \-#]+%?;)+">],
			   h1 => [qr<class="[a-z0-9_\-\ ]+">, qr<id="[a-z0-9_\-]+">, qr<align="[a-z]+">],
			   h2 => [qr<class="[a-z0-9_\-\ ]+">, qr<id="[a-z0-9_\-]+">, qr<align="[a-z]+">],
			   h3 => [qr<class="[a-z0-9_\-\ ]+">, qr<id="[a-z0-9_\-]+">, qr<align="[a-z]+">],
			   h4 => [qr<class="[a-z0-9_\-\ ]+">, qr<id="[a-z0-9_\-]+">, qr<align="[a-z]+">],
			   h5 => [qr<class="[a-z0-9_\-\ ]+">, qr<id="[a-z0-9_\-]+">, qr<align="[a-z]+">],
			   h6 => [qr<class="[a-z0-9_\-\ ]+">, qr<id="[a-z0-9_\-]+">, qr<align="[a-z]+">],
			   font => [qr<color="#?[a-fA-F0-9]+">, qr<size="[1-6]+">],
			   strong => [],
			   b => [],
			   p => [qr<class="[a-z0-9_\-\ ]+">, qr<id="[a-z0-9_\-]+">, qr<align="[a-z]+">],
			   br => [],
			   pre => [qr<class="[a-z0-9_\-\ ]+">, qr<id="[a-z0-9_\-]+">],
			   del => [qr<class="[a-z0-9_\-\ ]+">, qr<id="[a-z0-9_\-]+">],
			   ul => [qr<class="[a-z0-9_\-\ ]+">, qr<id="[a-z0-9_\-]+">, qr<type="[a-z]+">,
					  qr<data-role="[a-z0-9_\-]+">,
					  qr<data-url="<% url %>/(?:<% gmd %>|0)/[a-z0-9_]+\.json\?<% query\.html %>">,
					  qr<data-inset="(?:true|false)">, qr<data-theme="[a-z]">, qr<data-divider-theme="[a-z]">,
					  qr<data-split-icon="[a-z0-9_\-]+">, qr<data-split-theme="[a-z]">,
					  qr<data-filter="(?:true|false)">, qr<data-filter-placeholder="[^"]*">,
					  qr<data-filter-theme="[a-z]">],
			   ol => [qr<class="[a-z0-9_\-\ ]+">, qr<id="[a-z0-9_\-]+">, qr<type="[a-z]+">, qr<start="[0-9]+">,
					  qr<data-role="[a-z0-9_\-]+">,
					  qr<data-url="<% url %>/(?:<% gmd %>|0)/[a-z0-9_]+\.json\?<% query\.html %>">,
					  qr<data-inset="(?:true|false)">, qr<data-theme="[a-z]">, qr<data-divider-theme="[a-z]">,
					  qr<data-split-icon="[a-z0-9_\-]+">, qr<data-split-theme="[a-z]">,
					  qr<data-filter="(?:true|false)">, qr<data-filter-placeholder="[^"]*">,
					  qr<data-filter-theme="[a-z]">],
			   li => [qr<class="[a-z0-9_\-\ ]+">, qr<id="[a-z0-9_\-]+">, qr<value="[0-9]+">,
					  qr<data-role="[a-z0-9_\-]+">],
			   dl => [qr<class="[a-z0-9_\-\ ]+">, qr<id="[a-z0-9_\-]+">],
			   dt => [qr<class="[a-z0-9_\-\ ]+">, qr<id="[a-z0-9_\-]+">],
			   dd => [qr<class="[a-z0-9_\-\ ]+">, qr<id="[a-z0-9_\-]+">],
			   table => [qr<class="[a-z0-9_\-\ ]+">, qr<id="[a-z0-9_\-]+">,
						 qr<width="[0-9]+%?">, qr<cellspacing="[0-9]+">, qr<cellpadding="[0-9]+">,
						 qr<border="[0-9]+">, qr<bordercolor="#?[a-fA-F0-9]+">, qr<bgcolor="#?[a-fA-F0-9]+">,
						 qr<style="(?:[a-z\-]+:[a-zA-Z0-9 \-#]+%?;)+">],
			   'tr' => [qr<class="[a-z0-9_\-\ ]+">, qr<id="[a-z0-9_\-]+">, qr<align="[a-z]+">, qr<valign="[a-z]+">,
						qr<width="[0-9]+%?">, qr<height="[0-9]+">, qr<bgcolor="#?[a-fA-F0-9]+">],
			   th => [qr<class="[a-z0-9_\-\ ]+">, qr<id="[a-z0-9_\-]+">,
					  qr<align="[a-z]+">, qr<valign="[a-z]+">, qr<width="[0-9]+%?">, qr<height="[0-9]+">,
					  qr<colspan="[0-9]+">, qr<rowspan="[0-9]+">, qr<bgcolor="#?[a-fA-F0-9]+">,
					  qr<background="[a-z0-9_]+">],
			   td => [qr<class="[a-z0-9_\-\ ]+">, qr<id="[a-z0-9_\-]+">,
					  qr<align="[a-z]+">, qr<valign="[a-z]+">, qr<width="[0-9]+%?">, qr<height="[0-9]+">,
					  qr<colspan="[0-9]+">, qr<rowspan="[0-9]+">, qr<bgcolor="#?[a-fA-F0-9]+">,
					  qr<background="[a-z0-9_]+">],
			   a => [qr<class="[a-z0-9_\-\ ]+">, qr<id="[a-z0-9_\-]+">,
					 qr<href="(?:<% \$?(?:cgi_[a-z0-9_]+|[A-Z_]+|shp_url) %>|[a-zA-Z0-9_#])(?:<% \$?[a-z0-9_]+ %>|[a-zA-Z0-9_\.%&\/=~\-\?#;]+)*">,
					 qr<rel="external">, qr<data-role="[a-z0-9_\-]+">, qr<data-rel="[a-z]+">,
					 qr<data-inline="(?:true|false)">,
					 qr<data-transition="[a-z]+">, qr<data-direction="reverse">,
					 qr<data-theme="[a-z]">, qr<data-icon="[a-z0-9_\-]+">, qr<data-iconpos="[a-z]+">,
					 qr<data-page="[0-9]+">, qr<data-limit="[0-9]+">, qr<data-sort="[0-9]+">,
					 qr<name="[a-z0-9_]+">, qr<target="[a-z0-9_]+">],
			   img => [qr<src="(?:(?:<% [a-zA-Z0-9_]+ %>/)*[a-z0-9_][a-z0-9_\-/]+\.[a-zA-Z0-9_]+|<% shp_img[1-5]_file\.imgurl %>|<% img[1-6]_file\.imgurl %>)">,
					   qr<alt="[^"]+">, qr<align="[a-z]+">, qr<border="[0-9]+">,
					   qr<hspace="[0-9]+">, qr<vspace="[0-9]+">, qr<width="[0-9]+">, qr<height="[0-9]+">],
			   center => [],
			   hr => [qr<class="[a-z0-9_\-\ ]+">, qr<id="[a-z0-9_\-]+">,
					  qr<width="[0-9]+%?">, qr<size="[0-9]+">, qr<color="#?[a-fA-F0-9]+">, qr<align="[a-z]+">,
					  qr<style="(?:[a-z\-]+:[a-zA-Z0-9 \-#]+%?;)+">],
			   form => [qr<class="[a-z0-9_\-\ ]+">, qr<id="[a-z0-9_\-]+">, qr<method="(?:post|get)">,
						qr<name="[a-z0-9_]+">, qr<data-ajax="(?:true|false)">,
						qr<action="(?:<% [a-z0-9_]+ %>/)*(?:[a-z0-9_][a-z0-9_/]+\.[a-z0-9_]+)?">],
			   input => [qr<class="[a-z0-9_\-\ ]+">, qr<id="[a-z0-9_\-]+">,
						 qr<type="(?:hidden|text|search|range|checkbox|submit)">,
						 qr<name="[a-z0-9_]+">, qr<value="[^"]*">, qr<placeholder="[^"]*">,
						 qr<autocapitalize="(?:on|off)">, qr<autocorrect="(?:on|off)">,
						 qr<size="[0-9]+">, qr<maxlength="[0-9]+">, qr<data-theme="[a-z]">,
						 qr<style="(?:[a-z\-]+:[a-zA-Z0-9 \-#]+%?;)+">],
			   fieldset => [qr<data-role="[a-z0-9_\-]+">, qr<data-type="[a-z]+">],
			   legend => [],
			   label => [qr<for="[a-z0-9_\-]+">],
			  }
};
$C->{Sendmail}->{module} = 'Cozy::Sendmail';
$C->{Sendmail}->{config} = {
							type => 'command',
							x_mailer => $C->{NAME}.' '.$C->{VERSION},
							x_url => 'http://www.e-shoppy.com/',
							encoding => 'iso-2022-jp-ms',
							charset => 'iso-2022-jp',
							mime_encoding => 'MIME-Header-ISO_2022_JP_MS',
#							encoding => 'utf-8',
#							charset => 'utf-8',
#							mime_encoding => 'MIME-Header',
#							content_transfer_encoding => '8bit',
							debug => 0,
						   };
$C->{Session}->{module} = 'Cozy::Session';
$C->{Session}->{config} = {
						   valid_addr => 0,
						   valid_agent => 1,
						   expires => 60*60*12,
						  };
$C->{Cookie}->{module} = 'Cozy::CGI::Cookie';
$C->{Cookie}->{config} = {
						  debug => 0,
						 };
$C->{Exception}->{module} = 'Cozy::Exception';
$C->{Exception}->{config} = {};
$C->{Message}->{module} = 'Cozy::Message';
$C->{Message}->{config} = {};
$C->{Pager}->{module} = 'Cozy::Pager';
$C->{Pager}->{config} = { limit => 20 };
$C->{Calc}->{module} = 'ESP::SMS::Calc';
$C->{Calc}->{config} = { is_charged => 0 };
$C->{Search}->{module} = 'ESP::SMS::Pages::Users::Search';
$C->{Search}->{config} = { name => 'search', command => 1, cmd => 'ssi.html', ssl => 0 };
$C->{Articles}->{module} = 'ESP::SMS::Pages::Users::Articles';
$C->{Articles}->{config} = { name => 'articles', command => 1, cmd => 'ssi.html', ssl => 0 };
$C->{WebMgr}->{module} = 'ESP::SMS::Pages::Manage::WebMgr';
$C->{WebMgr}->{config} = { name => 'webmgr', command => 1, cmd => 'ssi.null', ssl => 0 };

# データアップロード用モジュール(拡張子別)
$C->{DataHandler}->{csv} = 'Cozy::DataHandler::File::CSV';
#$C->{DataHandler}->{xls} = 'ESP::SMS::DataHandler::File::Excel';

# CGI の名前と URL の設定
$C->{CGI}->{SCRIPT} = {
	#
	# '(CGIのファイル名)' => '(CGIのタイプ)(CGIのURL公開制限)(SSL対象のファイル);(説明)'
	#
	# CGIのタイプ       (0=一般用のファイル|1=管理用のファイル)
	# CGIのURL公開制限  (0=一般公開|1=非公開管理室用)
	# SSL対象のファイル (0=SSL使用不可|1=SSL使用許可)
	#
	'search' => '000;商品・ショップ検索表示用',
	'cart'	 => '001;買い物かご',
	'order'	 => '001;お会計処理用',
	'static' => '000;サイトページ表示用',
	'contact' => '001;問い合わせ/ショップ規約表示用',
	'customer' => '001;顧客情報表示用',
	'articles' => '000;記事検索表示用',
	'sitemap' => '000;サイトマップ表示用',

	'menu'	 => '101;ホーム管理室用',
	'sltmgr' => '111;決済方法管理用',
	'pstmgr' => '111;地域別送料管理用',
	'cmlmgr' => '111;通知メール管理用',
	'lawmgr' => '111;規約管理用',
	'psvmgr' => '111;決済サービス管理用',
	'webmgr' => '111;ファイル管理用',
	'docmgr' => '111;記事管理用',
	'shpmgr' => '111;ショップ管理用',
	'gdsmgr' => '111;登録管理用',
	'gnrmgr' => '111;ジャンル管理用',
	'grpmgr' => '111;カテゴリ管理用',
	'odrmgr' => '111;注文管理用',
	'ctmmgr' => '111;顧客管理用',
	'dmlmgr' => '111;メルマガ管理用',
	'chrmgr' => '111;利用料管理用',
	'sysmgr' => '111;システム管理用',
	'cfgmgr' => '111;モール設定用',
};

# データベースのタイプ
$C->{DB_TYPE} = 'Pg';

# テーブル名
$C->{DB_TABLE} = [qw(TRM ARE ANQ SHP LAW PSV PSD GDS GNR GRP CTE IMG REV TRK
					 CAT PAY DVY PST SLT FEE ADR CTM BIL BSC SEL ODR OSC EML
					 DML SML CML DOC DSK CHR ZIP CFG VIA LOG GDSZ)]; # TPL

# タイムアウト秒数(0の場合は制限なし)
$C->{TIMEOUT} = 0;

# デバック(0=無効/1=有効)
$C->{DEBUG} = 0;

__PACKAGE__->config($C);

sub make_config {
	my $self = shift;
	my $plugins = __PACKAGE__->config->{DIR_PLUGINS};
	print <<"EOF";
=== Make for $NAME $VERSION is now finished.
This CGI provides a configuration file at

$plugins/00config.pl

Edit to fit your needs.
EOF
}

sub make_finish {
	my $self = shift;
	print <<"EOF";

=== Edit your crontab by 'crontab -e', and put the following job.

0 * * * * $C->{DIR_LOCAL}/cron_t.pl
0 1 * * * $C->{DIR_LOCAL}/cron_d.pl
0 2 1 * * $C->{DIR_LOCAL}/cron_m.pl
0 4 * * * $C->{DIR_LOCAL}/backups.pl

=== Initial setup for $NAME $VERSION is now finished.
=== Thanks for using E-Shoppy software. Let's Start!
=== $C->{URL_BASE}/
EOF
}

1;
__END__
# Copyright (C) 2000-2012 E-Shoppy Project developing team
# All rights reserved.
