# -*- mode: cperl; coding: utf-8-unix; indent-tabs-mode: t -*-
# $Id: $

package ESP::SMS::Plugin::calendar;

use strict;
our $VERSION = '0.01';

use Cozy::DateTime;

my @WEEK = qw(sun mon tue wed thu fri sat);
my %WDAY = map { $WEEK[$_] => $_ } (0..6);

# このプラグインは、カレンダーを表示するフィルター<% .Calendar %>を追加します。
sub import {
	no strict 'refs';
	*{'ESP::SMS::Plugin::DataFilter::filter_Calendar'} = \&filter_calendar;
}

# テンプレート(LEFT.htmlなど)記述例:
#
# <div id="calendar">
# <% LOAD .Calendar(close=Sun,0101,0211,0321,0429,0503,0505,0923,1103,1123,1223&open=) -%>
# <%#
# 【休業日(close=)】
# ・休業日の曜日もしくは日付を指定して下さい。
# ・日付は、月2桁、日2桁を組み合わせた数字を使用して下さい。
# ・曜日は、Sun(日曜日)、Mon(月曜日)、Tue(火曜日)、Wed(水曜日)
#   Thu(木曜日)、Fri(金曜日)、Sat(土曜日)を使用して下さい。
# ・複数ある場合はコンマで区切って記述して下さい。
# 【例外営業日(open=)】
# ・指定した休業日の中で特別に営業する日を指定して下さい。
# ・日付は、月2桁、日2桁を組み合わせた数字を使用して下さい。
# ・複数ある場合はコンマで区切って記述して下さい。
# -%>
#
# <table class="calendar">
# <tr><th colspan="7" class="title"><% this_month %>月のカレンダー</th></tr>
# <tr><th>日</th><th>月</th><th>火</th><th>水</th><th>木</th><th>金</th><th>土</th></tr>
# <% FOR ThisMonth -%>
# <% IF wday.eq(0) %><tr>
# <% END -%>
# <% IF close %><td class="close"><% day %></td><% ELSE %><td class="open"><% day %></td><% END %>
# <% IF wday.eq(6) %></tr>
# <% END -%>
# <% END -%>
# </table>
#
# <table class="calendar">
# <tr><th colspan="7" class="title"><% next_month %>月のカレンダー</th></tr>
# <tr><th>日</th><th>月</th><th>火</th><th>水</th><th>木</th><th>金</th><th>土</th></tr>
# <% FOR NextMonth -%>
# <% IF wday.eq(0) %><tr>
# <% END -%>
# <% IF close %><td class="close"><% day %></td><% ELSE %><td class="open"><% day %></td><% END %>
# <% IF wday.eq(6) %></tr>
# <% END -%>
# <% END -%>
# </table>
# </div>
#
# CSS(common.css)記述例:
#
# div#calendar table {
#    width: auto;
#    margin-top: 5px;
#    margin-bottom: 5px;
#    margin-left: 10px;
#    border-collapse: collapse;
# }
# div#calendar table th, div#calendar table td {
#    padding: 1px 4px 1px 4px;
#    border: 1px solid #CCCCCC;
#    font-weight: normal;
#    text-align: right;
# }
# div#calendar table th {
#    color: #333333;
#    text-align: center;
#    background-color: #ABE670;
# }
# div#calendar table th.title {
#    color: #333333;
#    background-color: #FFFFFF;
# }
# div#calendar table td.close {
#    color: #B85651;
#    background-color: #FFCCCC;
# }

sub filter_calendar {
	my ($self, $stash, $args) = @_;
	my $opts = $self->parse_args(lc $args);
	# 営業日の集計
	my %bday;
	for my $param (split(/,/, $opts->{open}||"")) {
		if ($param =~ /^[a-z]{3}$/ && exists $WDAY{$param}) {
			$bday{'W'.$WDAY{$param}}++;
		}
		elsif ($param =~ /^[0-9]{4}$/) {
			$bday{$param}++;
		}
	}
	for my $param (split(/,/, $opts->{close}||"")) {
		if ($param =~ /^[a-z]{3}$/ && exists $WDAY{$param}) {
			$bday{'W'.$WDAY{$param}}--;
		}
		elsif ($param =~ /^[0-9]{4}$/) {
			$bday{$param}--;
		}
	}
	my $now = Cozy::DateTime->now;
	my ($year, $month, $day) = $now->ymd;
	# 今月の末日
	my $this_lday = $now->last_day;
	# 今月の開始時間
	my $this_time = $now->timelocal - (($day - 1) * 86400);
	my $this = Cozy::DateTime->new($this_time);
	# 来月の開始時間
	my $next_time = $this_time + ($this_lday * 86400);
	my $next = Cozy::DateTime->new($next_time);
	# 来月の末日
	my $next_lday = $next->last_day;
	my ($next_year, $next_month, $next_day) = $next->ymd;

	$stash->{ThisMonth} = mk_monthly_list($this, $this_lday, \%bday);
	$stash->{NextMonth} = mk_monthly_list($next, $next_lday, \%bday);
	$stash->{this_month} = $stash->{ThisMonth}->[0]->{month};
	$stash->{next_month} = $stash->{NextMonth}->[0]->{month};

	return $stash;
}

sub mk_monthly_list {
	my ($dt, $last_day, $bday) = @_;
	my ($year, $month) = $dt->ymd;
	my $wnum = $dt->wday;
	my $wday = 0;
	my @calendar;
	while ($wday < $wnum) {
		push @calendar, { year => $year, month => $month, wday => $wday, week => $WEEK[$wday] };
		$wday++;
	}
	for my $day (1..$last_day) {
		my $wday = $wnum++ % 7;
		my $date = sprintf("%02d%02d", $month, $day);
		# 指定休業日(祝日など)と指定休曜日が重なる場合は翌日を振替休業日にする。
		if (($bday->{$date}||0) < 0 && ($bday->{"W$wday"}||0) < 0) {
			$bday->{sprintf("%02d%02d", $month, $day+1)}--;
		}
		push @calendar, {
						 year => $year, month => $month, day => $day, wday => $wday, week => $WEEK[$wday],
						 close => ($bday->{$date}||$bday->{"W$wday"}||0) < 0 ? 1 : 0
						};
	}
	if ($wday = $wnum % 7) {
		while ($wday < 7) {
			push @calendar, { year => $year, month => $month, wday => $wday, week => $WEEK[$wday] };
			$wday++;
		}
	}
	return \@calendar;
}

1;
__END__
# Copyright (C) 2000-2012 E-Shoppy Project developing team
# All rights reserved.
