# -*- mode: cperl; coding: utf-8-unix; indent-tabs-mode: t -*-
# $Id: $

package ESP::SMS::Plugin::qrcode;

use strict;
our $VERSION = '0.01';

use ESP::SMS::Plugin;

ESP::SMS::Plugin->append('ESP::SMS::Pages::Manage::ShpMgr' => __PACKAGE__);

# このプラグインは、ショップのURLをQRコード(qrcode.gif)にして
# ショップディレクトリに追加します。
# 使用するには、ご利用のサーバにPerlのImager::QRCodeモジュールが
# インストールされている必要があります。

# 前処理
# ESP::SMS::Pages::Manage::ShpMgrが呼ばれた時に一度だけ実行されます。
sub _setup {
	my ($class, $pkg) = @_;
	require Imager::QRCode;
}

# ESP::SMS::Pages::Manage::ShpMgrにこのファイルで定義したメソッド
# dispatch_qrcodeを追加します。
# http://domain/manage/shpmgr/(ショップ番号)/qrcode.html
# 上記のURLでアクセスするとショップディレクトリにqrcode.gifが作成されます。
sub dispatch_qrcode {
	my $self = shift;
	my $tmpl = $self->tmpl;
	my $shop = $self->lookin;
	my $C = $self->plugin->config;
	my $filename = $C->{QRCODE_FILENAME}||'qrcode.gif';
	my $shp_id = $shop->shp_id;
	my ($shp_url, $img_url, $file);
	# ショップの場合
	if ($shp_id) {
		my $shp_dir = $shop->shp_dir
			or return $self->exception->throw({ level => 'error', code => 'P00001',
												id => 'shp_dir', retstr => 'No defined parameter: shp_dir' });
		$shp_url = $shop->shp_url
			or return $self->exception->throw({ level => 'error', code => 'P00002',
												id => 'shp_url', retstr => 'No defined parameter: shp_url' });
		$shp_url .= '/index.xhtml';
		$img_url = $C->{URL_STATIC}."/$shp_dir/$filename";
		$file = $C->{DIR_STATIC}."/$shp_dir/$filename";
	}
	# モール(shp_id=0)の場合
	else {
		$shp_url = $C->{URL_BASE}.'/';
		$img_url = $C->{URL_STATIC}."/$filename";
		$file = $C->{DIR_STATIC}.'/'.$filename;
	}
	# テンプレート変数の設定
	$tmpl->param(shp_url => $shp_url);
	$tmpl->param(img_url => $img_url);
	$tmpl->param(filename => $filename);
	$tmpl->param(time => time());
	# QRコード生成
	my $qrcode = Imager::QRCode->new(size => 3, margin => 3, version => 0, level => 'M',
									 casesensitive => 1,
									 lightcolor => Imager::Color->new(255, 255, 255),
									 darkcolor => Imager::Color->new(0, 0, 0));
	my $img = $qrcode->plot($shp_url);
	my $umask = umask();
	umask($C->{UMASK}) if defined $C->{UMASK};
	$img->write(file => $file) or do {
		umask($umask);
		return $self->exception->throw({ level => 'error', code => 'P00003',
										 id => $file, retstr => $img->errstr });
	};
	umask($umask);
	# テンプレート(plugins/skel/manage/shpmgr/_qrcode.html)
	# を作成した場合は以下は不要です。
	{
		use utf8;
		$tmpl->file(\<<'EOF');
<html>
<body>
QRコード(<% filename %>)が作成されました。<br />
<img src="<% img_url %>?<% time %>" /><br />
<% shp_url %>
</body>
</html>
EOF
	}
	return;
}

1;
__END__
# Copyright (C) 2000-2012 E-Shoppy Project developing team
# All rights reserved.
