# -*- mode: cperl; coding: utf-8-unix; indent-tabs-mode: t -*-
# $Id: $

package ESP::SMS::Plugin::thumbnail;

use strict;
our $VERSION = '0.02';

use ESP::SMS::Plugin;

ESP::SMS::Plugin->append('ESP::SMS::Pages::Manage::GdsMgr' => __PACKAGE__);
ESP::SMS::Plugin->append('ESP::SMS::Pages::Manage::ShpMgr' => __PACKAGE__);
ESP::SMS::Plugin->append('ESP::SMS::Pages::Manage::GnrMgr' => __PACKAGE__);
ESP::SMS::Plugin->append('ESP::SMS::Pages::Manage::GrpMgr' => __PACKAGE__);

# このプラグインの機能
# * 商品登録時に商品画像を自動的に縮小して登録します。
# * ショップ登録時にショップ画像を自動的に縮小して登録します。
# * ジャンル登録時にジャンル画像を自動的に縮小して登録します。
# * カテゴリ登録時にカテゴリ画像を自動的に縮小して登録します。
#
# ご利用条件
# * ご利用のサーバにPerlのImagerモジュールがインストールされている
#   必要があります。
#
# 変換サイズの設定
# 例) 商品画像1の幅を30ピクセルに変更する場合
# local/plugins/00config.pl に $C->{GDS_IMG_1_WIDTH} = 30; を追加するか
# あるいは以下の GDS_IMG_1_WIDTH => 60 を 30 に変更して下さい。
# ※0を指定すると変換されません。
#
# 標準の画像変換サイズ
my %DEFAULT = (
			   # 商品画像1の横幅
			   GDS_IMG_1_WIDTH => 60,
			   # 商品画像1の高さ
			   # GDS_IMG_1_HEIGHT => 0,
			   # 商品画像2の横幅
			   GDS_IMG_2_WIDTH => 60,
			   # 商品画像2の高さ
			   # GDS_IMG_2_HEIGHT => 0,
			   # 商品画像3の横幅
			   GDS_IMG_3_WIDTH => 60,
			   # 商品画像3の高さ
			   # GDS_IMG_3_HEIGHT => 0,
			   # 商品画像4の横幅
			   GDS_IMG_4_WIDTH => 120,
			   # 商品画像4の高さ
			   # GDS_IMG_4_HEIGHT => 0,
			   # 商品画像5の横幅
			   # GDS_IMG_5_WIDTH => 0,
			   # 商品画像5の高さ
			   # GDS_IMG_5_HEIGHT => 0,
			   # 商品画像6の横幅
			   # GDS_IMG_6_WIDTH => 0,
			   # 商品画像6の高さ
			   # GDS_IMG_6_HEIGHT => 0,
			   # ショップ画像1の横幅
			   # SHP_IMG_1_WIDTH => 0,
			   # ショップ画像1の高さ
			   SHP_IMG_1_HEIGHT => 50,
			   # ショップ画像2の横幅
			   # SHP_IMG_2_WIDTH => 0,
			   # ショップ画像2の高さ
			   SHP_IMG_2_HEIGHT => 100,
			   # ショップ画像3の横幅
			   # SHP_IMG_3_WIDTH => 0,
			   # ショップ画像3の高さ
			   # SHP_IMG_3_HEIGHT => 0,
			   # ジャンル画像1の横幅
			   # GNR_IMG_1_WIDTH => 0,
			   # ジャンル画像1の高さ
			   GNR_IMG_1_HEIGHT => 50,
			   # ジャンル画像2の横幅
			   # GNR_IMG_2_WIDTH => 0,
			   # ジャンル画像2の高さ
			   GNR_IMG_2_HEIGHT => 100,
			   # ジャンル画像3の横幅
			   # GNR_IMG_3_WIDTH => 0,
			   # ジャンル画像3の高さ
			   # GNR_IMG_3_HEIGHT => 0,
			   # カテゴリ画像1の横幅
			   # GRP_IMG_1_WIDTH => 0,
			   # カテゴリ画像1の高さ
			   GRP_IMG_1_HEIGHT => 50,
			   # カテゴリ画像2の横幅
			   # GRP_IMG_2_WIDTH => 0,
			   # カテゴリ画像2の高さ
			   GRP_IMG_2_HEIGHT => 100,
			   # カテゴリ画像3の横幅
			   # GRP_IMG_3_WIDTH => 0,
			   # カテゴリ画像3の高さ
			   # GRP_IMG_3_HEIGHT => 0,
			  );

# 前処理
# ESP::SMS::Pages::Manage::(GdsMgr|ShpMgr|GnrMgr|GrpMgr)が呼ばれた時に
# 一度だけ実行されます。
sub _setup {
	my ($class, $pkg) = @_;
	require Imager;
}

# 変換処理
# ESP::SMS::Pages::Manage::(GdsMgr|ShpMgr|GnrMgr|GrpMgr)内で呼出されます。
sub image_converter {
	my ($self, $file, $oid) = @_;
	my $class = ref($self) or return;
	my ($name) = $class =~ /::([a-zA-Z]+)Mgr$/ or return;
	my $C = $self->plugin->config;
	my $s = uc($name).'_IMG_'.$oid;
	my $width = exists $C->{"$s\_WIDTH"} ? $C->{"$s\_WIDTH"} : $DEFAULT{"$s\_WIDTH"}||0;
	my $height = exists $C->{"$s\_HEIGHT"} ? $C->{"$s\_HEIGHT"} : $DEFAULT{"$s\_HEIGHT"}||0;
	return unless $width || $height;
	my $img = Imager->new;
	$img->read(file => $file)
		or return $self->exception->throw({ level => 'error', code => 'P00001',
											id => $file, retstr => $img->errstr });
	my ($suffix) = $file =~ /\.([a-zA-Z0-9_]+)$/ or return;
	my $tmpfile = "$file.$suffix";
	unlink $tmpfile if -e $tmpfile;
	my %scale;
	$scale{xpixels} = $width if $width;
	$scale{ypixels} = $height if $height;
	$img = $img->scale(%scale);
	my $umask = umask();
	umask($C->{UMASK}) if defined $C->{UMASK};
	$img->write(file => $tmpfile, jpegquality => 95)
		or umask($umask),
			return $self->exception->throw({ level => 'error', code => 'P00002',
											 id => $file, retstr => $img->errstr });
	umask($umask);
	unlink $file;
	rename $tmpfile, $file
		or return $self->exception->throw({ level => 'error', code => 'P00003',
											id => $tmpfile, retstr => $! });
	return 1;
}

1;
__END__
# Copyright (C) 2000-2012 E-Shoppy Project developing team
# All rights reserved.
