/* -*- coding: utf-8-unix; indent-tabs-mode: t -*-
 *
 * ImageView v1.0
 * by (C) 2009-2010 E-Shoppy Project
 * required: jQuery v1.3
 * Licensed under both MIT and GPL License
 *
 * CSS:
 * div.ImageView {
 *     background-color: #EEEEEE;
 *     padding: 10px;
 *     border: 1px solid #CCCCCC;
 * }
 * div.ImageView div.ImageViewClose {
 *     background-color: #880000;
 *     color: #FFFFFF;
 * }
 * div.ImageView div.ImageViewClose a {
 *     color: #FFFFFF;
 * }
 * div.ImageViewLoading {
 *     padding: 10px;
 *     border: 1px solid #CCCCCC;
 *     background-color: #FFCCCC;
 *     color: #AA3333;
 * }
 * 
 * HTML:
 * <script type="text/javascript" src="/path/to/jquery.js"></script>
 * <script type="text/javascript" src="/path/to/imageview.js"></script>
 * <script type="text/javascript">
 * <!--
 * $(function() {
 *    $('a.zoom').ImageView();
 * });
 * //-->
 * </script>
 * 
 * <a href="/path/to/image1.jpg" class="zoom">Image1</a>
 * <a href="/path/to/image2.jpg" class="zoom">Image2</a>
 * <a href="/path/to/image3.jpg" class="zoom">Image3</a>
 * 
 */
$.fn.ImageView = function(config){
	config = $.extend({ label: '画像を閉じる', loadmsg: '画像を読み込んでいます...',
						zindex: 10, regexp: /\.jpg$|\.jpeg$|\.png$|\.gif$/ }, config);
	var label = config.label;
	var loadmsg = config.loadmsg;
	var zindex = config.zindex;
	var regexp = config.regexp;
	return this.each(function(){
		$(this).click(function(elm){
			var url = $(this).attr('href');
			if (url == undefined || url.toLowerCase().match(regexp) == null) {
				$(this).attr('target', '_blank');
				return true;
			}
			var id = 'ImageView'+(new Date().getTime());
			var target = '#'+id;
			var image = new Image();
			var doc = document;
			var centerX = (doc.body.scrollLeft||doc.documentElement.scrollLeft)+elm.clientX;
			var centerY = (doc.body.scrollTop||doc.documentElement.scrollTop)+elm.clientY;
			var loading = null;
			if (loadmsg != '') {
				$(target+'-loading').hide();
				$('body').append('<div id="'+id+'-loading" class="ImageViewLoading">'+loadmsg+'</div>');
				loading = $(target+'-loading');
				loading.css('position', 'absolute').css('z-index', zindex++)
					.css('left', centerX).css('top', centerY).show();
			}
			image.onload = function(){
				if (loading != null) loading.remove();
				var imageX = centerX - image.width / 2;
				var imageY = centerY - image.height / 2;
				if (imageX < 0) imageX = 5;
				if (imageY < 0) imageY = 5;
				var dialog = ['<div id="', id,
							  '" class="ImageView"><div class="ImageViewClose"><a href="javascript:void(0);">',
							  label, '</a></div><img src="', url+'?'+id,
							  '"><div class="ImageViewClose"><a href="javascript:void(0);">',
							  label, '</a></div></div>'].join('');
//							  '<iframe id="', id, '" class="ImageView"></iframe>'].join('');
				$(target).hide();
				$('body').append(dialog);
				var selector = $(target); // 再取得
				selector.css('position', 'absolute').css('z-index', zindex++)
					.css('left', imageX).css('top', imageY)
					.find('div.ImageViewClose').click(function(e){
						selector.remove();
						return false;
				});
				selector.mousedown(function(e){
					var left = parseInt(selector.css('left'), 10);
					var top = parseInt(selector.css('top'), 10);
					var startX = e.clientX;
					var startY = e.clientY;
					selector.css('z-index', zindex++).css('cursor', 'move');
					selector.mousemove(function(e){
						selector.css('left', left + e.clientX - startX).css('top', top + e.clientY - startY);
						return false;
					});
					selector.mouseup(function(e){
						selector.css('left', left + e.clientX - startX).css('top', top + e.clientY - startY)
							.css('cursor', 'default').unbind('mousemove').unbind('mouseup');
						return false;
					});
					return false; // バブリング停止
				});
				selector.show();
			};
			image.src = url;
			return false;
		});
	});
};
