/* -*- coding: utf-8-unix; indent-tabs-mode: t -*-
 *
 * (C) 2009-2012 E-Shoppy Project
 * required: jQuery Mobile v1.0rc2
 *
 */

$(document).bind("mobileinit", function(){
	$.mobile.ajaxEnabled = false; 
	/* 日本語化 */
	$.mobile.loadingMessage = '読込み中';
	$.mobile.pageLoadErrorMessage = '読込みに失敗しました';
	$.mobile.page.prototype.options.backBtnText = '戻る';
	$.mobile.dialog.prototype.options.closeBtnText = '閉じる';
	$.mobile.selectmenu.prototype.options.closeText= '閉じる';
	$.mobile.listview.prototype.options.filterPlaceholder = '検索文字列...';
	/* selectタグでカスタムメニューを使用する。*/
	/* 個別に設定する場合は <select data-native-menu="false"> */
	$.mobile.selectmenu.prototype.options.nativeMenu = false;
});

(function ($, undefined) {

/*
 * 商品のスライド作成
 * required: jquery.touchSlider.js v1.0
 *
 * 使用例:
 *
 * <script type="text/javascript">
 * <!--
 * $(function() {
 *    $('.carousel').Carousel({ mode: 'index', center: true, single: true });
 * });
 * //-->
 * </script>
 *
 * <div class="carousel">
 * <div class="holder">
 * <div class="list">
 *    <div class="item"><div class="item-inner">item1</div></div>
 *    <div class="item"><div class="item-inner">item2</div></div>
 *    <div class="item"><div class="item-inner">item3</div></div>
 * </div>
 * </div>
 * <span class="prev">&lt;</span>
 * <span class="next">&gt;</span>
 * <div class="tabs"></div>
 * </div>
 *
 */
var CarouselDefaults = {
	item: 'div.item',
	holder: 'div.holder',
	box: 'div.list',
	prevLink: 'span.prev',
	nextLink: 'span.next',
	tabs: 'div.tabs',
	itemInner: 'div.item-inner',
	itemInnerWidthOffset: 40
};
$.fn.Carousel = function(options){
	options = $.extend(true, {}, CarouselDefaults, options);
	return this.each(function(){
		var gallery = this;
		var tabs = $(gallery).find(options.tabs).first();
		var prev = $(gallery).find(options.prevLink).get(0);
		var next = $(gallery).find(options.nextLink).get(0);
		if (options.mode == 'index' && options.single) {
			var itemInnerWidthOffset = options.itemInnerWidthOffset || 0;
			var itemInnerWidth = $(gallery).find(options.holder).first().width() - itemInnerWidthOffset;
			$(gallery).find(options.itemInner).css('max-width', itemInnerWidth);
		}
		$(gallery).touchSlider($.extend({}, options, {
			prevLink: prev,
			nextLink: next,
			onChange: function(prev, curr) {
				$(gallery).find('a.tablink').removeClass('active');
				$(gallery).find('a.tablink').filter(function(i){return i == curr}).addClass('active');
			},
			onStart: function() {
				var count = gallery.getCount();
				tabs.html('');
				for (var i = 0; i < count; i++) {
					var el = $('<a href="javascript:void(0);" class="tablink">'+(i+1)+'</a>');
					el.attr('index', i);
					tabs.append(el);
					el.bind('click', function(){
						gallery.moveTo($(this).attr('index'));
						return false;
					});
				}
			}
		}));
	});
};

/*
 * 次のページのデータ取得表示
 *
 * 実行例
 *
 * <ul data-role="listview" data-url="http://www.yourdomain.co.jp/search/0/index.json" id="list">
 * <li>商品1 100円</li>
 * <li>商品2 100円</li>
 * </ul>
 *
 * <script type="text/javascript">//<![CDATA[
 * $(function(){
 *    $('#list').Pager({ id: '#pager', render: function(data) {
 *    var html = [];
 *    html.push('<li>'+data.gds_name+' '+data.gds_price+'円</li>');
 *    html.push("\n");
 *    return html.join('');
 *    }});
 * });
 * //]]>
 * </script>
 *
 * <div class="pager">
 * <a href="javascript:void(0);" rel="external" data-page="2" data-limit="2" data-sort="0" id="pager">次の2件...</a>
 * </div>
 *
 */
$.fn.Pager = function(options){
	var options = $.extend({ id: '#pager', render: function(){} }, options);
	var id = options.id;
	var render = options.render;
	var list = this;

	$(id).bind('click', function(){
		var link = $(this);
		var url = link.attr('data-url')||list.attr('data-url')||'';
		var pg = link.attr('data-page')||1;
		var li = link.attr('data-limit')||20;
		var so = link.attr('data-sort')||0;
		if (url == '') return false;
		var query = 'pg='+pg+'&li='+li+'&so='+so;
		url += url.indexOf('?') > -1 ? '&'+query : '?'+query;
		$.ajax({
			type: 'GET',
			dataType: 'json',
			url: url,
			async: true,
			cache: false,
			timeout: 10000, /* 10秒 */
			beforeSend: function(){
				$.mobile.showPageLoadingMsg(); // loadingメッセージを表示
			},
			success: function(json, status){
				var data = json.data;
				var N = data.length;
				for (var i = 0; i < N; i++) {
					list.append(render(data[i]));
				}
				var next_page = json.next_page;
				if (next_page == '') {
					$(id).hide();
				} else {
					link.attr('data-page', next_page);
				}
				list.listview('refresh');
			},
			complete: function(){
				$.mobile.hidePageLoadingMsg(); // loadingメッセージを隠す
			}
		});
		return false;
	});
};

/*
 * 住所データの取得表示
 *
 * 実行例
 *
 * <!-- main -->
 * <script type="text/javascript">//<![CDATA[
 * $(function(){
 *    $('#PostalList').PostalSearch({
 *       search: 'a.PostalSearch',
 *       url: 'http://domain.com/contact/0/address.json',
 *       area: '#area',
 *       city: '#city',
 *       render: function(data) {
 *          var html = [];
 *          html.push('<li><a href="javascript:void(0);">');
 *          html.push(data.zip_code+'<br />');
 *          html.push(data.zip_are_name+' ');
 *          html.push(data.zip_city_name+' ');
 *          html.push(data.zip_town_name);
 *          html.push('</a></li>');
 *          html.push("\n");
 *          return html.join('');
 *       }
 *    });
 * });
 * //]]>
 * </script>
 *
 * <form>
 * <input type="text" value="" id="PS_code" />
 * <a href="#postal" rel="" data-role="button" data-rel="dialog" data-transition="pop" id="PS" class="PostalSearch">住所入力</a>
 * <select name="adr_are_id" id="PS_area">
 * <option value="" selected="selected"></option>
 * <option value="1">北海道</option>
 * </select>
 * <input type="text" name="adr_city" value="" id="PS_city">
 * <input type="text" name="adr_town" value="" id="PS_town">
 * </form>
 *
 * <!-- dialog -->
 * <div data-role="page" id="postal">
 * <div data-role="header" data-theme="d">
 *    <h3>郵便番号・住所の検索</h3>
 * </div>
 *
 * <div data-role="content" data-theme="c">
 * <h4>都道府県</h4>
 * <select name="zip_are_id" id="area">
 * <option value="" selected="selected"></option>
 * <option value="1">北海道</option>
 * </select>
 *
 * <h4>市区町村郡</h4>
 * <select name="zip_city_id" id="city">
 * <option value="" selected="selected"></option>
 * </select>
 *
 * <h4>検索結果</h4>
 * <ul data-role="listview" data-divider-theme="c" id="PostalList">
 * </ul>
 * </div>
 * </div>
 *
 */
$.fn.PostalSearch = function(options){
	var options = $.extend({ search: '.PostalSearch', url: '',
							 area: '#area', city: '#city', prepare: null, render: null }, options);
	var list = this;
	var url = options.url;
	if (url == '') return false;
	var Target;
	$(options.search).bind('click', function(){ // 住所入力クリック時
		var link = $(this);
		Target = link;
		var id = link.attr('id');
		var code = $('#'+id+'_code');
		var zip_code = new String(code.val());
		if (!zip_code.match(/^\d\d\d\d\d\d\d$/)) {
			return true;
		}
		PostalList({
			url: url,
			query: 'zip_code='+zip_code,
			list: list,
			prepare: options.prepare,
			render: options.render
		});
		return true;
	});
	$(options.city).bind('change', function(){ // 市区町村選択時
		var link = $(this);
		var zip_are_id = new String($(options.area).val());
		var zip_city_id = new String(link.val());
		if (!zip_are_id.match(/^\d+$/) || !zip_city_id.match(/^\d+$/)) {
			return true;
		}
		PostalList({
			url: url,
			query: 'zip_are_id='+zip_are_id+'&zip_city_id='+zip_city_id,
			list: list,
			prepare: options.prepare,
			render: options.render
		});
		return true;
	});
	function PostalList(opt){
		$.ajax({
				type: 'GET',
				dataType: 'json',
				url: opt.url.indexOf('?') > -1 ? opt.url+'&'+opt.query : opt.url+'?'+opt.query,
				async: true,
				cache: false,
				timeout: 10000, // 10秒
				beforeSend: function(){
					$.mobile.showPageLoadingMsg(); // loadingメッセージを表示
				},
				success: function(json, status){
					var data = json.data;
					if ($.isFunction(opt.prepare)) {
						opt.prepare(data);
					}
					if ($.isFunction(opt.render)) {
						opt.list.html('');
						var N = data.length;
						for (var i = 0; i < N; i++) {
							var item = data[i];
							var html = $(opt.render(item));
							html.find('a').first()
								.attr('data-zip_code', item.zip_code)
								.attr('data-zip_are_id', item.zip_are_id)
								.attr('data-zip_city_name', item.zip_city_name)
								.attr('data-zip_town_name', item.zip_town_name)
								.bind('click', function() { PostalInput(this) });
							opt.list.append(html);
						}
						opt.list.listview('refresh');
					}
				},
				complete: function(){
					$.mobile.hidePageLoadingMsg(); // loadingメッセージを隠す
				}
		});
	}
	function PostalInput(eml){
		var a = $(eml);
		var id = Target.attr('id');
		$('#'+id+'_code').val(a.attr('data-zip_code'))
		$('#'+id+'_area').val(a.attr('data-zip_are_id')).selectmenu('refresh');
		$('#'+id+'_city').val(a.attr('data-zip_city_name'));
		$('#'+id+'_town').val(a.attr('data-zip_town_name'));
		$(Target.attr('href')).dialog('close');
		return false;
	}
};

/*
 * 都道府県から市区町村取得表示
 *
 * 実行例
 *
 * <script type="text/javascript">//<![CDATA[
 * $(function(){
 *    $('#area').PostalSearchFromAreaToCity({ url: 'http://domain.com/contact/0/city.json', city: '#city' });
 * });
 * //]]>
 * </script>
 *
 * <h4>都道府県</h4>
 * <select name="zip_are_id" id="area">
 * <option value="" selected="selected"></option>
 * <option value="1">北海道</option>
 * </select>
 *
 * <h4>市区町村郡</h4>
 * <select name="zip_city_id" id="city">
 * <option value="" selected="selected"></option>
 * </select>
 *
 */
$.fn.PostalSearchFromAreaToCity = function(options){
	options = $.extend({ url: '', city: '#city' }, options);
	var url = options.url;
	var city = options.city;
	return this.each(function(){
		return $(this).bind('change', function(){
			var form = this.form;
			var zip_are_id = $(this).val();
			$.ajax({
				type: 'GET',
				dataType: 'json',
				url: url+'?zip_are_id='+zip_are_id,
				async: true,
				cache: false,
				beforeSend: function(){
					$.mobile.showPageLoadingMsg(); // loadingメッセージを表示
				},
				success: function(json, status){
					var data = json.data;
					var sel = $(city).get(0);
					sel.options[0] = new Option('選択して下さい', '');
					sel.length = 1;
					var N = data.length;
					for (var i = 0; i < N; i++) {
						sel.options[i+1] = new Option(data[i].label, data[i].id);
					}
					sel.length = N + 1;
					$(city).selectmenu('refresh');
				},
				complete: function(){
					$.mobile.hidePageLoadingMsg(); // loadingメッセージを隠す
				}
			});
			return false;
		});
	});
};

/*
 * パスワード作成
 *
 * 使用例:
 *
 * <script type="text/javascript">
 * <!--
 * $(function() {
 *    $('input.passgen').PassGenerator({ digit: 10 });
 * });
 * //-->
 * </script>
 *
 * <form action="/path/to" method="post">
 * パスワード1:<input name="field1" type="password" value="" size="20" maxlength="20" />
 * <input type="button" value="パスワード1作成" id="passgen-field1" class="passgen" />
 *
 * パスワード2:<input name="field2" type="password" value="" size="20" maxlength="20" />
 * <input type="button" value="パスワード2作成" id="passgen-field2" class="passgen" />
 * </form>
 *
 */
$.fn.PassGenerator = function(options){
	options = $.extend({ confirm: "次のパスワードを入力しますか？\n"+
						 "パスワードをメモしてから確認(OK)ボタンを押して下さい。",
						 chars: "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_-",
						 digit: 8 }, options);
	var chars = options.chars; // 使用文字
	var D = options.digit; // パスワードの桁数
	var N = chars.length;
	return this.each(function(){
		$(this).bind('click', function(){
			var c = [];
			for (var i = 0; i < D; i++) c.push(chars.charAt(Math.floor(Math.random() * N)));
			var passwd = prompt(options.confirm, c.join(''));
			if (passwd != null && passwd != "") {
				var id = $(this).attr('id').split('-');
				$(this.form).find('input[name='+id[1]+']').val(passwd);
			}
			return false;
		});
	});
};

})(jQuery);
