/* -*- coding: utf-8-unix; indent-tabs-mode: t -*-
 *
 * Validator v1.0
 * by (C) 2009-2010 E-Shoppy Project
 * required: jQuery v1.3
 * Licensed under both MIT and GPL License
 *
 * HTML:
 * <script type="text/javascript" src="/path/to/jquery.js"></script>
 * <script type="text/javascript" src="/path/to/validator.js"></script>
 * <script type="text/javascript">
 * <!--
 * $('#mainform').submit(function(){
 *    return $(this).Validate({
 *       fields: {
 *          subject: { desc: '件名', type: ['required'] },
 *          summary: { desc: '記事概要', type: ['required'], maxlength: 512 },
 *          message: { desc: '記事詳細', maxlength: 15360 }
 *       }
 *    });
 * });
 * //-->
 * </script>
 * 
 * <form action="/path/to/post" method="post" id="mainform">
 * <input type="text" name="subject" size="60" value="" />
 * <textarea name="summary" cols="80" rows="5"></textarea>
 * <textarea name="message" cols="80" rows="20"></textarea>
 * <input type="submit" value="登録" />
 * </form>
 * 
 */
$.Validator = function(desc, errors){
	// 値の型
	this.desc = desc;
	this.errors = errors;
	this.required = function(val){
		if (val != '') return true;
		this.errors.push(this.desc+'を入力して下さい。');
		return false;
	};
	this.chars = function(val){
		if (val == '' || val.match(/^[a-z]+$/)) return true;
		this.errors.push(this.desc+'は半角文字(a〜z、A〜Z、0〜9、_)を入力して下さい。');
		return false;
	};
	this.number = function(val){
		if (val == '' || val.match(/^\d+$/)) return true;
		this.errors.push(this.desc+'は数字を入力して下さい。');
		return false;
	};
	this.date = function(val){
		if (val == '' || val.match(/^\d\d\d\d\/\d\d?\/\d\d?$/)) return true;
		this.errors.push(this.desc+'は日付(YYYY/MM/DD形式)を入力して下さい。');
		return false;
	};
	this.datetime = function(val){
		if (val == '' || val.match(/^\d\d\d\d\/\d\d?\/\d\d? \d\d:\d\d(:\d\d)?$/)) return true;
		this.errors.push(this.desc+'は日時(YYYY/MM/DD HH:MM:SS形式)を入力して下さい。');
		return false;
	};
	// 値の範囲
	this.minnumber = function(val, length){
		var num = new Number(val);
		if (num >= length) return true;
		this.errors.push(this.desc+'は'+length+'以上を入力して下さい。');
		return false;
	};
	this.maxnumber = function(val, length){
		var num = new Number(val);
		if (num <= length) return true;
		this.errors.push(this.desc+'は'+length+'以下を入力して下さい。');
		return false;
	};
	this.minlength = function(val, length){
		if (val.length >= length) return true;
		this.errors.push(this.desc+'は'+length+'文字以上で記述して下さい。(現在の文字数:'+val.length+'文字)');
		return false;
	};
	this.maxlength = function(val, length){
		if (val.length <= length) return true;
		this.errors.push(this.desc+'は'+length+'文字以内で記述して下さい。(現在の文字数:'+val.length+'文字)');
		return false;
	};
};

$.InputData = function(input){
	this.range = ['minlength', 'maxlength', 'minnumber', 'maxnumber'];
	this.errors = [];
	this.input = input;
	this.format = input.format;
	this.validator = new $.Validator('▲'+input.format.desc, this.errors);
	this.validate_type = function(){
		var type = this.input.type;
		if (type == undefined) return this;
		var N = type.length;
		var val = this.input.val;
		for (var i = 0; i < N; i++) {
			var method = type[i];
			if (!this.validator[method](val)) break;
		}
		return this;
	};
	this.validate_range = function(){
		var val = this.input.val;
		if (val == '') return this;
		var N = this.range.length;
		for (var i = 0; i < N; i++) {
			var method = this.range[i];
			if (this.format[method] == undefined) continue;
			if (!this.validator[method](val, this.format[method])) break;
		}
		return this;
	};
	this.update_class = function(){
		var input = this.input.obj;
		var css = input.attr('class');
		if (this.errors.length > 0) {
			if (css == 'str') {
				input.attr('class', 'error_str');
			} else if (css == 'chr') {
				input.attr('class', 'error_chr');
			}
		} else {
			if (css == 'error_str') {
				input.attr('class', 'str');
			} else if (css == 'error_chr') {
				input.attr('class', 'chr');
			}
		}
		return this;
	};
	this.error_messages = function(){
		return this.errors.join("\n");
	};
	this.has_errors = function(){
		return this.errors.length > 0 ? true : false;
	};
};

$.fn.Validate = function(options){
	options = $.extend({ suffix: false,	fields: {}, title: '登録内容' }, options);
	var is_valid = true;
	var fields = options.fields;
	$(this).each(function(index){
		var errors = [];
		var suffix = options.suffix ? '_'+(index+1) : '';
		var form = $(this);
		var input = {};
		var field;
		for (field in fields) {
			var name = field+suffix;
			var obj = form.find('input[name='+name+']:first,textarea[name='+name+']:first,select[name='+name+']:first');
			var format = fields[field];
			var val = obj.val();
			if (val == undefined) {
				alert(field+': field not found!');
				return false;
			}
			// デフォルト値
			if (val == '' && format.value != undefined) {
				obj.val(val);
			}
			if (format.type == undefined) format.type = [];
			input[field] = { obj: obj, val: val, format: format, type: format.type.slice() };
		}
		// 依存項目
		for (field in input) {
			var depend_fields = input[field].format.depend_fields;
			if (depend_fields != undefined && input[field].val != '') {
				var N = depend_fields.length;
				for (var i = 0; i < N; i++) {
					var df = depend_fields[i];
					if (input[df].type[0] != 'required') input[df].type.unshift('required');
				}
			}
		}
		for (field in input) {
			var data = new $.InputData(input[field]);
			data.validate_type();  // 値の型
			data.validate_range(); // 値の範囲
			data.update_class();   // エラーの場合はクラス名変更
			if (data.has_errors()) errors.push(data.error_messages());
		}
		if (errors.length > 0) {
			if (options.suffix) {
				alert(options.title+'['+(index+1)+']に誤りがあります。'+"\n\n"+errors.join("\n"));
			} else {
				alert(options.title+'に誤りがあります。'+"\n\n"+errors.join("\n"));
			}
			is_valid = false;
			if (suffix != '') return false;
		}
	});
	return is_valid;
};
